#!/bin/bash

## Define default value for variable, take two arguments, $1 variable name,
## $2 default variable value, if the variable is not already define define it
## with default value.
function define_default_value()
{
	VAR_NAME="${1}"
	DEFAULT_VALUE="${2}"

	[ -z "${!VAR_NAME}" ] && export ${VAR_NAME}="${DEFAULT_VALUE}"
}

define_default_value WORK_DIR "$(mktemp --directory)/"
define_default_value CLEANUP_WORKDIR true
define_default_value TAR_FILE "RetroShare.tar.gz"
define_default_value SRC_DIR "$(realpath $(dirname $BASH_SOURCE)/../../)"
SRC_DIR="$(realpath "$SRC_DIR")" # Enforce absolute path if passed
define_default_value PACKAGING_VERSION "unstable"

ORIG_DIR="$(pwd)"

function git_check_submodule()
{
	pSubmodulePath="$1" ; shift
	pRepoPath="${1-$SRC_DIR}" ; shift

	[ "$(ls "${pRepoPath}/${pSubmodulePath}" | wc -l)" -lt "2" ] &&
	{
		# --force removed because can interfere with local modifications
		# --remote removed because doesn't honor the version indicated in the
		#	main repository enforcing checking out last version of the submodule
		#	potentially breaking everything
		git -C "${pRepoPath}" submodule update \
			--depth 1 --init \
			"$pSubmodulePath"
	}
}

# if SRC_DIR does not exists chcekout clean RetroShare source
if [ ! -d "$SRC_DIR" ]; then
	# Full clone so we have the tags too
	git clone git@github.com:RetroShare/RetroShare.git "$SRC_DIR"
	git_check_submodule build_scripts/OBS || exit -2
fi

git_check_submodule supportlibs/restbed
git_check_submodule dependency/asio "${SRC_DIR}/supportlibs/restbed"
git_check_submodule dependency/catch "${SRC_DIR}/supportlibs/restbed"
git_check_submodule dependency/kashmir "${SRC_DIR}/supportlibs/restbed"

git_check_submodule supportlibs/udp-discovery-cpp
git_check_submodule supportlibs/rapidjson
git_check_submodule supportlibs/libsam3
git_check_submodule supportlibs/cmark
git_check_submodule supportlibs/jni.hpp

git_check_submodule libbitdht
git_check_submodule libretroshare
git_check_submodule openpgpsdk
git_check_submodule retroshare-webui

pushd "${WORK_DIR}"
rsync -a --delete \
	--exclude='**.git*' \
	--exclude='/build_scripts/OBS/network:retroshare/**/.osc**' \
	--exclude='/build_scripts/OBS/network:retroshare/**/binaries**' \
	--exclude='/build_scripts/OBS/network:retroshare/**/*.tar.gz' \
	--exclude='*.a' --exclude='*.so' --exclude='*.o' --exclude='**~' \
	--exclude='*.pro.user' \
	--exclude='.gradle/' \
	--exclude='**.kdev4' \
	--exclude='/.kdev4/**' \
	--exclude='CMakeLists.txt.user' \
	--include='/supportlibs/libsam3/Makefile' \
	--exclude='Makefile**' \
	"${SRC_DIR}/" RetroShare/

## Source_Version File
#RE_VERSION='s/^[[:alpha:]](.*)-g.*$/\1/g'
RE_VERSION='s/^[[:alpha:]]//g'
VERSION="$(git -C ${SRC_DIR} describe)"
DEBVERSION=`echo $VERSION | sed -E "$RE_VERSION" | sed "s/-/./g"`
echo ${VERSION} > RetroShare/Source_Version
cat RetroShare/Source_Version

## Update Debian ChangeLog
DEB_CHGLOG_GUI="RetroShare/build_scripts/OBS/network:retroshare/retroshare-gui-unstable/debian.changelog"
DEB_CHGLOG_CMN="RetroShare/build_scripts/OBS/network:retroshare/retroshare-common-unstable/debian.changelog"
>${DEB_CHGLOG_GUI}
function logentry() {
	local version=$1
	local describe=`git -C ${SRC_DIR} describe ${version} | sed -E "$RE_VERSION"`
	echo "retroshare-gui-unstable ($describe) unstable; urgency=low"
	echo
	git -C ${SRC_DIR} show $version --quiet --oneline --format="  * %s:%b"
	echo
	git -C ${SRC_DIR} show $version --quiet --oneline --format=" -- %an <%ae>  %aD"
	echo
}
git -C ${SRC_DIR} log -10 --pretty=format:%H | (
	while read version; do
		logentry $version >> ${DEB_CHGLOG_GUI}
	done
)
sed "s/retroshare-gui-unstable/retroshare-common-unstable/g" ${DEB_CHGLOG_GUI} > ${DEB_CHGLOG_CMN}

## Debian Description
DEB_DESCR_GUI="RetroShare/build_scripts/OBS/network:retroshare/retroshare-gui-unstable/retroshare-gui-unstable.dsc"
DEB_DESCR_CMN="RetroShare/build_scripts/OBS/network:retroshare/retroshare-common-unstable/retroshare-common-unstable.dsc"
sed -i "s/Version: 0.6.9999/Version: ${DEBVERSION}-1/g" ${DEB_DESCR_GUI}
sed -i "s/Version: 0.6.9999/Version: ${DEBVERSION}-1/g" ${DEB_DESCR_CMN}

## openSUSE:Specfile
OBS_SPEC_GUI="RetroShare/build_scripts/OBS/network:retroshare/retroshare-gui-unstable/retroshare-gui-unstable.spec"
sed -i "s/Version:       0.6.9999/Version:       ${DEBVERSION}/g" ${OBS_SPEC_GUI}


echo "Making Archive ..."
tar -zcf ${TAR_FILE} RetroShare/
SIZE="$(wc -c ${TAR_FILE} | awk '{ print $1 }')"
MD5="$(md5sum ${TAR_FILE} | awk '{ print $1 }')"
echo ""
echo "MD5                              Size     Name"
echo "${MD5} ${SIZE} ${TAR_FILE}"
mv ${TAR_FILE} "${ORIG_DIR}/${TAR_FILE}"

$CLEANUP_WORKDIR && rm -rf "${WORK_DIR}"

echo "Preparation for git version ${VERSION} and debian ${DEBVERSION} finished."

[ "$SIZE" -ge "50000000" ] &&
{
	echo "${TAR_FILE} is $SIZE bigger the 50MB some error must have happened!"
	exit -1
}
